#!/usr/bin/python
# -*- coding: UTF-8 -*-

import os
import openpyxl
import xlrd
import xlwt

class Excel:
    _wb = None
    _sheet = None
    _file_name = False
    _keys = []
    _excel_type = 'xls'

    # 初始化
    def __init__(self, file="", hasKey=True, sheet_name=""):
        if file == "":
            return None

        self._file_name = file
        self._excel_type = 'xlsx' if ".xlsx" in self._file_name else self._excel_type

        if os.path.isfile(self._file_name):
            if self._excel_type == 'xlsx':
                self._wb = openpyxl.load_workbook(self._file_name)
                self._sheet = self._wb.active
            else:
                # self._wb = self._open_xls_as_xlsx(self._file_name)
                self._wb = xlrd.open_workbook(self._file_name)
                self._sheet = self._wb.sheets()[0]
        else:
            if self._excel_type == 'xlsx':
                self._wb = openpyxl.Workbook()
                self._sheet = self._wb.active
            else:
                self._wb = xlwt.Workbook()
                if sheet_name == "":
                    sheet_name = 'Sheet1'
                self._sheet = self._wb.add_sheet(sheet_name)

        # self._sheet = self._wb.active if self._sheet == None else self._sheet
        if hasKey:
            self.parse_key()

    # 解析标题栏
    def parse_key(self):
        start = 1 if self._excel_type == 'xlsx' else 0
        for index in range(self.get_cols()):
            self._keys.append(self._sheet.cell(start, index+start).value)

    def get_titles(self):
        return self._keys

    # 根据行/列读写数据
    def column(self, row, col, value=None):
        # col_letter = get_column_letter(col)
        if value == None:
            result = self._sheet.cell(row, col).value
            if type(result) is type(''):
                result = result.replace('_x000D_','')
            return result

        try:
            if self._excel_type == "xlsx":
                self._sheet.cell(row, col).value = value
            else:
                self._sheet.write(row-1, col-1, value)
        except:
            pass

    # 根据行及key读取数据
    def get_value(self, row, key, default=""):
        row = row if self._excel_type == "xlsx" else row -1

        try:
            if key in self._keys:
                col = self._keys.index(key) + 1 if self._excel_type == "xlsx" else self._keys.index(key)
                row += 1

                result = self._sheet.cell(row, col).value
                if type(result) is type(''):
                    result = result.replace('_x000D_','')
                
                if default == 0:
                    return 0 if result == "" else int(result)
                if default == "":
                    return "" if result == None else str(result)

                return result
        except:
            pass
        
        return default
        
    # 设置行列数据
    def set_value(self, row, key, value):
        if key in self._keys:
            col = self._keys.index(key) + 1
            self._sheet.cell(row, col).value = value
            return True

        return False

    # 获取当前工作表的标题
    def title(self):
        return self._sheet.title

    # 获取当前工作表行数
    def get_rows(self):
        if self._excel_type == "xlsx":
            return self._sheet.max_row - 1 if self._sheet.max_row > 0 else 0
        else:
            return self._sheet.nrows - 1 if self._sheet.nrows > 0 else 0

    # 获取当前工作表列数
    def get_cols(self):
        if self._excel_type == 'xlsx':
            return self._sheet.max_column
        else:
            return self._sheet.ncols

    # 存储Excel
    def save(self):
        self._wb.save(self._file_name)

    # 打开xls转化为xlsx-暂时弃用
    def _open_xls_as_xlsx(self, filename):
        # first open using xlrd
        xls_book = xlrd.open_workbook(filename)
        sheet = xls_book.sheets()[0]
        nrows = sheet.nrows
        ncols = sheet.ncols

        # 读取多个sheet，暂时不支持
        # index = 0
        # nrows, ncols = 0, 0
        # while nrows * ncols == 0:
        #     sheet = xls_book.sheet_by_index(index)
        #     nrows = sheet.nrows
        #     ncols = sheet.ncols
        #     index += 1

        # prepare a xlsx sheet
        xlsx_book = openpyxl.Workbook()
        xlsx_sheet = xlsx_book.get_active_sheet()

        for row in range(0, nrows):
            for col in range(0, ncols):
                xlsx_sheet.cell(row+1, col+1).value = sheet.cell_value(row, col)

        return xlsx_book



#!/usr/bin/python
# -*- coding: UTF-8 -*-

from kinglib.excel import Excel

class Excelfile:

    def __init__(self, excel_file, headers=[]):
        self._sheet = Excel(excel_file)
        self._headers = self._sheet.get_titles() if len(headers) == 0 else headers
        self._row = 1
        

    def _write_table_title(self):
        for key in self._headers:
            self._sheet.column(1, self._get_column_id(key), key)

    def _get_column_id(self, column_name):
        try:
            return self._headers.index(column_name) + 1
        except:
            return 0

    def write_excel(self, values):
        # 写表头
        if self._row == 1:
            self._write_table_title()
            self._row += 1

        # 写数据
        for index, key in enumerate(values):
            self._sheet.column(self._row, self._get_column_id(key), values[key])
            
        self._row += 1

        return self._row

    def get_row_value(self, row, key):
        return self._sheet.get_value(row, key)

    def get_title(self):
        return self._headers

    def get_rows(self):
        return self._sheet.get_rows()

    def close(self):
        self._sheet.save()


#!/usr/bin/python
# -*- coding: UTF-8 -*-
import time
import hmac
import hashlib
import base64
import urllib, json
from urllib.request import Request, urlopen

class Dingtalk:

    _secret = ""
    _access_token = ""
    _send_api = "https://oapi.dingtalk.com/robot/send?access_token="
    _headers = {"Content-Type": "application/json"}

    """
    初始化，创建群机器人时获取secret及access_token

    secret: 加签串，验签方式保证接口安全
    access_token: 访问token
    """
    def __init__(self, secret="", access_token=""):
        self._secret = secret
        self._access_token = access_token
    
    """
    获取时间戳及签名串
    """
    def _get_timestamp_sign(self):

        timestamp = int(round(time.time() * 1000))
        secret_enc = bytes(self._secret, 'utf-8')
        string_to_sign = '{}\n{}'.format(timestamp, self._secret)
        string_to_sign_enc = bytes(string_to_sign, 'utf-8')

        hmac_code = hmac.new( secret_enc, string_to_sign_enc, digestmod=hashlib.sha256).digest()
        sign = urllib.parse.quote_plus(base64.b64encode(hmac_code))

        return timestamp, sign

    """
    生成消息发送URL
    """
    def _get_dingtalk_send_api(self):
        timestramp, sign = self._get_timestamp_sign()
        return "{}{}&timestamp={}&sign={}".format(self._send_api, self._access_token, timestramp, sign)

    """
    发送文本消息

    message: 要发送的文本内容
    atMobiles: 需at提醒的钉钉用户手机号列表
    isAtall: 是否at提醒所有群用户
    """
    def send_text(self, message, atMobiles=[], isAtAll=False):
        raw = {"msgtype": "text", "text": {"content": message}, "at": {"atMobiles": atMobiles, "isAtAll": isAtAll}}
        return self._send_message(raw)
    
    """
    发送MarkDown文本消息

    message: 要发送的文本内容
    atMobiles: 需at提醒的钉钉用户手机号列表
    isAtall: 是否at提醒所有群用户
    """
    def send_markdown(self, title, content, atMobiles=[], isAtAll=False):
        raw = {"msgtype": "markdown", "markdown": {"title": title, "text": content}, "at": {"atMobiles": atMobiles, "isAtAll": isAtAll}}
        return self._send_message(raw)

    """
    发送图文链接

    title: 图文链接标题
    content: 图文摘要
    picUrl: 图文中图片链接
    messageUrl: 图文链接目标地址
    """
    def send_link(self, title, content, picUrl, messageUrl):
        raw = {"msgtype": "link", "link": {"text": content, "title": title, "picUrl": picUrl, "messageUrl": messageUrl}}
        return self._send_message(raw)
        
    
    """
    发送ActionCard消息

    title: 信息标题
    content: 信息内容
    btns: 按钮组信息，支持一个按钮和多个按钮；title-按钮方案，actionURL-点击按钮触发的URL
    btnOrientation: 0-按钮竖直排列，1-按钮横向排列
    hideAvatar: 0-正常发消息者头像，1-隐藏发消息者头像
    """
    def send_action_card(self, title, content, btns=[], btnOrientation="0", hideAvatar="0"):
        raw = {
                "msgtype": "actionCard",
                "actionCard": {
                    "title": title,
                    "text": content,
                    "hideAvatar": hideAvatar,
                    "btnOrientation": btnOrientation
                }
            }

        if len(btns) == 1:
            raw["actionCard"]["singleTitle"] = btns[0]["title"]
            raw["actionCard"]["singleURL"] = btns[0]["actionURL"]
        else:
            raw["actionCard"]['btns'] = btns

        return self._send_message(raw)

    """
    发送feed链接组

    links: feed组
        title: 信息标题
        picURL: 信息图片URL
        messageURL: 信息目标链接地址
    """
    def send_feed_card(self, links=[]):
        raw = {
                "msgtype": "feedCard",
                "feedCard": {
                    "links": links
                }
            }
        return self._send_message(raw)

    """
    消息体发送

    json_data: 所要发送的消息体
    """
    def _send_message(self, json_data={}):
        url = self._get_dingtalk_send_api()
        data = bytes(json.dumps(json_data), "utf-8")
        request = Request(url, headers={"Content-Type": "application/json"}, data=data)
        response = urlopen(request)
        html = response.read().decode("utf-8")
        result = json.loads(html)
        
        return result['errcode'], result["errmsg"]





#!/usr/bin/python
# -*- coding: UTF-8 -*-

from setuptools import setup, find_packages

setup(
    name='kinglib',
    version='0.1.0',
    keywords=('setup', 'kinglib'),
    description='setup kinglib',
    long_description='', # open('README.md').read()
    license='MIT',
    install_requires=[
        'openpyxl>=3.0.3',
        'xlrd>=1.2.0',
        'xlwt>=1.3.0'
    ],
    author='Guozhi Hu',
    author_email='guozhi.hu@qq.com',
    packages=find_packages(),
    platforms='any',
    url='',
    include_package_data = True,
    entry_points={
        'console_scripts':[
            'kinglib=kinglib.run:main' 
        ]
    },
    zip_safe=False
)